/************************************************************************
* "Epsilon" is a registered trademark licensed to Lugaru Software, Ltd. *
*		"EEL" and "Lugaru" are trademarks of Lugaru Software, Ltd.		*
*																		*
*     Copyright (C) 1996 Lugaru Software Ltd.  All rights reserved.	    *
*																		*
* Limited permission is hereby granted to reproduce and modify this		*
* copyrighted material provided that the resulting code is used only in *
* conjunction with Lugaru products and that this notice is retained in	*
* any such reproduction or modification.								*
************************************************************************/

#include "eel.h"
#include "proc.h"
#include "colcode.h"
#include "c.h"

// HTML syntax highlighting for Epsilon.
// By Lugaru Software, Ltd.  November 1996

// First define the color classes we'll use, and the color selections
// for each color scheme.

// Good defaults for other color schemes:
// html_text: text
// html_tag: c_function or c_keyword
// html_attribute: c_string
// html_value: c_preprocessor

color_scheme "standard-gui" {
	color_class html_text black;
	color_class html_tag blue;
	color_class html_attribute dark_magenta;
	color_class html_value dark_grey;
};
color_scheme "visual-c" {
	color_class html_text black;
	color_class html_tag dark_grey;
	color_class html_attribute dark_magenta;
	color_class html_value blue;
};
color_scheme "other-grey" {
	color_class html_text black;
	color_class html_tag blue;
	color_class html_attribute dark_red;
	color_class html_value dark_cyan;
};
color_scheme "standard-color" {
	color_class html_text white;
	color_class html_tag yellow;
	color_class html_attribute cyan;
	color_class html_value dark_cyan;
};
color_scheme "night-light" {
	color_class html_text white;
	color_class html_tag blue;
	color_class html_attribute dark_green;
	color_class html_value cyan;
};
color_scheme "yellow-on-blue" {
	color_class html_text yellow;
	color_class html_tag white;
	color_class html_attribute cyan;
	color_class html_value green;
};
color_scheme "white-on-blue" {
	color_class html_text white;
	color_class html_tag yellow;
	color_class html_attribute cyan;
	color_class html_value green;
};
color_scheme "blue-on-cyan" {
	color_class html_text dark_grey;
	color_class html_tag blue;
	color_class html_attribute dark_magenta;
	color_class html_value black;
};
color_scheme "white-background" {
	color_class html_text blue;
	color_class html_tag black;
	color_class html_attribute dark_magenta;
	color_class html_value dark_green;
};
color_scheme "dos-edit" {
	color_class html_text grey;
	color_class html_tag yellow;
	color_class html_attribute cyan;
	color_class html_value green;
};
color_scheme "borlandc" {
	color_class html_text yellow;
	color_class html_tag white;
	color_class html_attribute cyan;
	color_class html_value green;
};
color_scheme "grey-background" {
	color_class html_text black;
	color_class html_tag blue;
	color_class html_attribute dark_cyan;
	color_class html_value dark_grey;
};

// functions for coloring HTML code

color_html_from_here(safe)	// Move backward to the nearest line guaranteed
{			// to start outside any colored region, return pos.
	safe = safe;		// Prevent warning msg.
	return 0;		// (HTML coloring is fast; just start from the top.)
}

color_html_attributes()
{
	int s;

	while (s = parse_string(1, "[-_/a-z0-9]*")) {	// An attribute?
		set_character_color(point, point + s, color_class html_attribute);
		point += s;			// Followed by a value?
		if (s = parse_string(1, "[ \t\n]*=[ \t\n]*")) {
			set_character_color(point, point + s, color_class html_text);
			point += s;		// Value might be quoted.
			s = point;
			if (curchar() == '"') {
				point++;
				search(1, "\"");
			} else if (curchar() == '\'') {
				point++;
				search(1, "'");
			} else	// No quotes, assume it's whitespace-delimited.
				re_search(1, "[^ \t\n<>]*");
			set_character_color(s, point, color_class html_value);
		}
		re_search(1, "[ \t\n]*");	// Parse whitespace.
		set_character_color(matchstart, matchend, color_class html_text);
	}
}

color_html_range(from, to) // recolor just this section
{			// last colored region may go past to
	int s;

	if (from >= to)
		return to;
	save_var point, matchstart, matchend;
	if (from < to)
		set_character_color(from, to, color_class html_text);
	point = from;
	save_var case_fold = 1;
	for (s = point; point < to && search(1, "<"); s = point) {
		re_search(1, "[ \n]*");		// Parse < followed by whitespace.
		set_character_color(s, point, color_class html_text);
		if (s = parse_string(1, "[-_/a-z0-9]*")) {	// Parse the tag.
			set_character_color(point, point + s, color_class html_tag);
			point += s;
			re_search(1, "[ \t\n]*");	// Parse whitespace.
			set_character_color(matchstart, matchend,
								color_class html_text);
			color_html_attributes();
		}
	}
	return point;
}

char _html_mode_name[] = "HTML";
keytable html_tab;			/* key table for html mode */

command html_mode()
{
	mode_keys = html_tab;		/* Use these keys. */
	major_mode = _html_mode_name;

	strcpy(comment_start, "%<%!-- *");
	strcpy(comment_pattern, "%<%!--(.|<newline>)*--%><FirstEnd>");
	strcpy(comment_begin, "<!-- ");
	strcpy(comment_end, " -->");
	recolor_range = color_html_range;	// set up coloring rules
	recolor_from_here = color_html_from_here;
	if (want_code_coloring)		// maybe turn on coloring
		when_setting_want_code_coloring();
	try_calling("html-mode-hook");
	make_mode();
}

// Automatically go into HTML mode on .htm, .html files.

suffix_htm()
{
	html_mode();
}

suffix_html()
{
	html_mode();
}
